from django.core.exceptions import ImproperlyConfigured, PermissionDenied
from icontract import ViolationError


class UndeletableModelError(PermissionDenied):
    pass


class CustomTransactionExceptionWithCode(ImproperlyConfigured):
    code = None

    def __init__(self, message, *args, **kwargs):
        if self.code is not None:
            message = f"[{self.code}] {message}"
        super().__init__(message, *args, **kwargs)


class ActiveModelClassMustBeTransactionBackedError(CustomTransactionExceptionWithCode):
    code = "TE001"


class CannotReuseExistingTransactionError(CustomTransactionExceptionWithCode):
    code = "TE002"


class MissingTransactionInModelError(CustomTransactionExceptionWithCode):
    code = "TE003"


class SentinelTransactionCannotBeUsedError(CustomTransactionExceptionWithCode):
    code = "TE004"


class CustomTransactionBackedExceptionWithCode(PermissionDenied):
    code = None

    def __init__(self, message, *args, **kwargs):
        if self.code is not None:
            message = f"[{self.code}] {message}"
        super().__init__(message, *args, **kwargs)


class NotAnAnchorError(CustomTransactionBackedExceptionWithCode):
    code = "TBE001"


# Custom exception class for a specific type of contract violation
class TransactionTypeError(ViolationError):
    """
    use this exception to indicate that a transaction is not Transation type
    and inside an icontract require clause

    Example:
    >>> @require(
    >>>  lambda transaction=None: transaction is None or isinstance(transaction, Transaction),
    >>>  description="'transaction' in kwargs must be an instance of 'Transaction'.",
    >>>  error=TransactionTypeError  # Use your custom exception here
    >>> )
    >>> def save_project_fas(
    >>>   *,
    >>>   project: Union[Project, int],
    >>>   fas: Union[Fas, int],
    >>>   transaction: Union[Transaction, None] = None,
    >>>   **kwargs
    >>> ):
    """

    pass



from django.db import models

from django_anchor_modeling.fields import BusinessIdentifierField
from django_anchor_modeling.models import CreatedModel


class BusinessToDataFieldMapAbstract(models.Model):
    id = BusinessIdentifierField(primary_key=True)
    description = models.TextField()
    map = models.JSONField(default=dict)

    class Meta:
        abstract = True


class BusinessToDataFieldMap(CreatedModel):
    id = BusinessIdentifierField(primary_key=True)
    description = models.TextField()
    map = models.JSONField(default=dict)

    # in the form of "app_label.model_class_name"
    main_model_class = models.CharField(max_length=255, null=True)


class BusinessToQueryMap(CreatedModel):
    id = BusinessIdentifierField(primary_key=True)
    description = models.TextField()
    # in the form of "app_label.model_class_name"
    main_model_class = models.CharField(max_length=255)
    # e.g. the related names of the main model
    # ["quotation", "po_type", "description"]
    # ["parent", "parent__grandparent","parent__grandparent__greatgrandparent"]
    # ancestors of the main_model_class in one to one or many to one way
    # where main_model_class is the one
    select_related = models.JSONField(default=list)
    # descendants of main_model_class where it's one to many or many to many
    # [
    #     {
    #         "prefetch_field": "parent_set",
    #         "model": "myapp.Parent",
    #         "nested_prefetch": [
    #             {
    #                 "prefetch_field": "child_set",
    #                 "model": "myapp.Child",
    #                 "nested_prefetch": [
    #                     {
    #                         "prefetch_field": "grandchild_set",
    #                         "model": "myapp.Grandchild"
    #                     }
    #                 ]
    #             }
    #         ]
    #     }
    # ]
    prefetch_related = models.JSONField(default=list)
    # e.g. the fields in the related models
    # ["id", "quotation__value", "description__value"]
    only = models.JSONField(default=list)
    # field processors
    # where the key is the business level name of the field
    # and the value is a function that takes in the value of the field
    # {
    #     "quotation": {
    #         "field": "quotation",
    #         "condition": "hasattr",
    #         "attribute": "quotation",
    #         "sub_field": "value.display_quotation_number"
    #     },
    #     "another_field": {
    #         "field": "another_field",
    #         "condition": "hasattr",
    #         "attribute": "related_attribute",
    #         "sub_field": "value.some_other_property"
    #     }
    # }
    field_processors = models.JSONField(default=dict)



# Generated by Django 4.2.6 on 2023-10-25 18:18

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("metadata", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="datachange",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="functioncall",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="requestcall",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]
